﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using ZionFoundation.Components;
using ZionFoundation.GlobalTasks;

namespace ZionRobot.Models.Items
{
	// Token: 0x02000034 RID: 52
	internal class Folder : Folder
	{
		// Token: 0x06000189 RID: 393 RVA: 0x00008ED4 File Offset: 0x000070D4
		private void StartMonitoring()
		{
			this.FolderMonitor = new FolderMonitoring(base.FullPath, new FileSystemEventHandler(this.Monitor_Trigger));
			bool monitor = base.Monitor;
			if (monitor)
			{
				this.FolderMonitor.Initilize(true);
			}
		}

		// Token: 0x0600018A RID: 394 RVA: 0x00008F1C File Offset: 0x0000711C
		public override void Monitor_Trigger(object sender, FileSystemEventArgs e)
		{
			bool flag = !base.ExtensionsToIgnore.Contains(new FileInfo(e.FullPath).Extension);
			if (flag)
			{
				bool flag2 = System.IO.File.Exists(e.FullPath);
				if (flag2)
				{
					base.Append(this.Build(e.FullPath, base.Monitor));
				}
			}
		}

		// Token: 0x0600018B RID: 395 RVA: 0x00008F78 File Offset: 0x00007178
		public IComponent Build(string path, bool monitor = true)
		{
			FileAttributes attr = System.IO.File.GetAttributes(path);
			bool flag = (attr & FileAttributes.Directory) == FileAttributes.Directory;
			IComponent result;
			if (flag)
			{
				result = new Folder(path, monitor);
			}
			else
			{
				result = new ZionRobot.Models.Items.File(path);
			}
			return result;
		}

		// Token: 0x0600018C RID: 396 RVA: 0x00008FB0 File Offset: 0x000071B0
		public Folder(string path, bool monitor = true) : base(path, monitor)
		{
			base.Status = (monitor ? "1" : "0");
			base.Monitor = monitor;
			base.ExtensionsToIgnore = new string[]
			{
				".xml",
				".ztd",
				".vrf",
				".db",
				".~tmp"
			};
			this.StartMonitoring();
			base.LoadContent();
		}

		// Token: 0x1700009A RID: 154
		// (get) Token: 0x0600018D RID: 397 RVA: 0x00009028 File Offset: 0x00007228
		public new List<ZionRobot.Models.Items.File> Files
		{
			get
			{
				bool flag = this.neasted == null;
				if (flag)
				{
					this.neasted = new List<IComponent>();
				}
				this.CheckFiles();
				return (from item in this.neasted.ToList<IComponent>()
				where item.GetType() != base.GetType()
				select (ZionRobot.Models.Items.File)item into x
				orderby x.Name
				select x).ToList<ZionRobot.Models.Items.File>();
			}
		}

		// Token: 0x1700009B RID: 155
		// (get) Token: 0x0600018E RID: 398 RVA: 0x000090C4 File Offset: 0x000072C4
		public new List<Folder> Folders
		{
			get
			{
				bool flag = this.neasted == null;
				if (flag)
				{
					this.neasted = new List<IComponent>();
				}
				this.CheckFolders();
				return (from item in this.neasted.ToList<IComponent>()
				where item.GetType() == base.GetType()
				select (Folder)item into x
				orderby x.Name
				select x).ToList<Folder>();
			}
		}

		// Token: 0x0600018F RID: 399 RVA: 0x00009160 File Offset: 0x00007360
		public override List<IComponent> LoadFiles()
		{
			List<IComponent> components = new List<IComponent>();
			System.IO.Directory.GetFiles(base.FullPath).ToList<string>().ForEach(delegate(string x)
			{
				bool flag = !x.EndsWith(".xml") && !x.EndsWith(".ztd") && !x.EndsWith(".vrf") && !x.EndsWith("Thumbs.db") && !x.EndsWith(".~tmp");
				if (flag)
				{
					components.Add(new ZionRobot.Models.Items.File(x));
				}
			});
			return components;
		}

		// Token: 0x06000190 RID: 400 RVA: 0x000091AC File Offset: 0x000073AC
		public override List<IComponent> LoadFolders()
		{
			List<IComponent> components = new List<IComponent>();
			System.IO.Directory.GetDirectories(base.FullPath).ToList<string>().ForEach(delegate(string x)
			{
				components.Add(new Folder(x, this.Monitor));
			});
			return components;
		}

		// Token: 0x06000191 RID: 401 RVA: 0x00009200 File Offset: 0x00007400
		private void CheckFiles()
		{
			List<ZionRobot.Models.Items.File> existingFiles = (from child in this.neasted.ToList<IComponent>()
			where child.GetType() != this.GetType()
			select child into s
			select (ZionRobot.Models.Items.File)s).ToList<ZionRobot.Models.Items.File>();
			System.IO.Directory.GetFiles(base.FullPath).ToList<string>().ForEach(delegate(string x)
			{
				bool flag = !x.EndsWith(".xml") && !x.EndsWith(".ztd") && !x.EndsWith(".vrf") && !x.EndsWith("Thumbs.db") && !x.EndsWith(".~tmp");
				if (flag)
				{
					bool flag2 = (from p in existingFiles
					where p.FullPath == x
					select p).Count<ZionRobot.Models.Items.File>() == 0;
					if (flag2)
					{
						this.neasted.Add(new ZionRobot.Models.Items.File(x));
					}
				}
			});
		}

		// Token: 0x06000192 RID: 402 RVA: 0x00009288 File Offset: 0x00007488
		private void CheckFolders()
		{
			List<Folder> existingFolders = (from child in this.neasted
			where child.GetType() == this.GetType()
			select child into s
			select (Folder)s).ToList<Folder>();
			System.IO.Directory.GetDirectories(base.FullPath).ToList<string>().ForEach(delegate(string x)
			{
				bool flag = (from p in existingFolders
				where p.FullPath == x
				select p).Count<Folder>() == 0;
				if (flag)
				{
					this.neasted.Add(new Folder(x, this.Monitor));
				}
			});
		}

		// Token: 0x04000097 RID: 151
		private FolderMonitoring FolderMonitor;
	}
}
