﻿using System;
using System.Collections.Generic;
using ZionVault.Basic;

namespace ZionVault.Binder.OCR
{
    public class Area : IDisposable
    {
        private int index;
        private Location position;
        private List<Paragraph> paragraphes;

        public void Dispose()
        {
            if (this.paragraphes != null)
                foreach (Paragraph p in paragraphes)
                    p.Dispose();
            GC.SuppressFinalize(this);
        }

        public int Index
        {
            get { return index; }
            set { index = value; }
        }

        public Location Location
        {
            get { return position; }
            set { position = value; }
        }

        public List<Paragraph> Paragraphes
        {
            get { return paragraphes; }
            set { paragraphes = value; }
        }

        public Area(int _Index, Location _Posision, List<Paragraph> _Paragraphes)
        {
            this.index = _Index;
            this.position = _Posision;
            this.paragraphes = _Paragraphes;
        }

        public Area(System.Xml.XmlElement node)
        {
            this.paragraphes = new List<Paragraph>();

            this.index = int.Parse(node.GetAttribute(@"index"));
            this.position = new Location(node.GetAttribute(@"location"));
            foreach (System.Xml.XmlElement paragraph in node.GetElementsByTagName(@"paragraph"))
            {
                this.paragraphes.Add(new Paragraph(paragraph));
            }
        }

        public System.Xml.XmlNode Export(System.Xml.XmlDocument context)
        {
            System.Xml.XmlElement root = context.CreateElement(@"area");
            root.SetAttribute(@"index", this.index.ToString());
            root.SetAttribute(@"location", this.position.ToString());

            foreach (Paragraph element in this.paragraphes)
            {
                root.AppendChild(element.Export(context));
            }

            return root;
        }
    }
}
