﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace ZionVault.Binder.Verified
{
    public class Page : IDisposable
    {
        public void Dispose()
        {
            if (this.SearchElements != null)
                foreach (Field f in SearchElements)
                    f.Dispose();

            GC.SuppressFinalize(this);
        }

        public List<Field> SearchElements { get; set; }

        public List<Field> Header
        {
            get
            {
                return (from Field field in SearchElements
                        where field.FullName.StartsWith(@"SearchElements.Header")
                        select field).ToList();
            }
        }

        public List<Field> Footer
        {
            get
            {
                return (from Field field in SearchElements
                        where field.FullName.StartsWith(@"SearchElements.Footer")
                        select field).ToList();
            }
        }

        public List<Field> Fields
        {
            get
            {
                return (from Field field in SearchElements
                        where field.FullName.StartsWith(@"SearchElements.Fields")
                        select field).ToList();
            }
        }

        public string FullText { get; set; }

        public int Rotation { get; set; }

        public int Index { get; set; }

        public string Status { get; set; }

        public string Type { get; set; }

        public Page()
        {
            SearchElements = new List<Field>();
            Rotation = 0;
            Status = string.Empty;
            Type = string.Empty;
            FullText = string.Empty;
        }

        public Page(string text)
        {
            this.FullText = text;
            SearchElements = new List<Field>();
            Rotation = 0;
            Status = string.Empty;
            Type = string.Empty;
        }

        public Page(XmlElement element)
        {
            SearchElements = new List<Field>();
            Type = element.Attributes[@"Type"]?.Value ?? string.Empty;
            Index = int.Parse(element.Attributes[@"Index"].Value);
            Rotation = int.Parse(element.Attributes[@"Rotation"].Value);
            Status = element.Attributes[@"Status"]?.Value ?? string.Empty;

            foreach (XmlElement field in element.SelectSingleNode(@"fields"))
            {
                SearchElements.Add(new Field(field));
            }
        }

        public XmlElement Export(XmlDocument context)
        {
            XmlElement tag = context.CreateElement(@"page");
            XmlElement fields = context.CreateElement(@"fields");

            tag.SetAttribute(@"Type", Type);
            tag.SetAttribute(@"Index", Index.ToString());
            tag.SetAttribute(@"Rotation", Rotation.ToString());
            tag.SetAttribute(@"Status", Status);

            foreach (Field field in SearchElements)
                if (field.Exportable)
                    fields.AppendChild(field.Export(context));

            tag.AppendChild(fields);
            return tag;
        }
    }
}
